/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 6 mars 2006
   derniere modification le 19/10/2007 */

#ifndef __SCESHADERS_H__
#define __SCESHADERS_H__

#include <SCE/core/SCECShader.h>

/* constantes propres a Shaders */
#define SCE_UNKNOWN_SHADER -1 /* hum... */
#define SCE_ASM_SHADER 0
#ifdef _SCE_USE_CG
 #define SCE_CG_SHADER 1
#endif
#define SCE_GLSL_SHADER 2

#define SCE_SHADER_BAD_INDEX -1

typedef struct sce_sshader SCE_SShader;
struct sce_sshader
{
    void *v, *p;                 /* vertex/pixel shader */
    SCE_CProgram *p_glsl;        /* program GLSL */

    unsigned int type;           /* type du shader (langage; GLSL, ASM, Cg) */
    char **srcs;                 /* sources du/des fichier(s) (ressource) */
    char *vs_source, *ps_source; /* sources principales */
    char *vs_addsrc, *ps_addsrc; /* code source additif (defines, ...) */
    int ready;                   /* true; le shader peut etre utilise */
};


/* initialise le gestionnaire de shaders */
int SCE_Init_Shader (void);
/* quitte le gestionnaire de shaders */
void SCE_Quit_Shader (void);

/* retourne l'identifiant de type du media manager */
int SCE_Shader_GetMediaTypeID (void);

/* cree un shader */
SCE_SShader* SCE_Shader_Create (int);
/* detruit un shader */
void SCE_Shader_Delete (SCE_SShader*);

/* retourne le langage du shader (Cg, GLSL ou ASM) */
int SCE_Shader_GetLanguage (SCE_SShader*);

/* retourne le type du shader (vertex, pixel, ou les deux) */
int SCE_Shader_GetType (SCE_SShader*);



/* charge (char**) un fichier contenant le code du vertex shader
   et/ou le code du pixel shader (prefixes : [Vertex Shader] [Pixel Shader]) */
void* SCE_Shader_LoadSourceFromFile (FILE*, const char*, void*);

/* cree un shader a partir de deux fichiers, le premier contient le code du
   vertex shader, le second le code du pixel shader. un des deux fichiers peut
   contenir les deux types de code ensemble */
SCE_SShader* SCE_Shader_CreateFromFile (const char*, const char*);

/* construit un shader */
int SCE_Shader_Build (SCE_SShader*);

/* ajoute un morceau de code source qui sera ajoute
   au debut du code source par defaut */
int SCE_Shader_AddSource (SCE_SShader*, int, char*);


/* retourne l'index d'une variable de shader */
int SCE_Shader_GetIndex (SCE_SShader*, int, const char*);
/*
int SCE_Shader_GetParam(SCE_SShader*, const char*);
float SCE_Shader_GetParamf(SCE_SShader*, const char*);
float* SCE_Shader_GetParamfv(SCE_SShader*, const char*);
*/

void SCE_Shader_Param (int, const char*, int);
void SCE_Shader_Paramf (int, const char*, float);
void SCE_Shader_Paramfv (int, const char*, size_t, float*);

void SCE_Shader_SetParam (int, int);
void SCE_Shader_SetParamf (int, float);
void SCE_Shader_SetParamfv (int, size_t, float*);


/* defini si la modification de shader via Use est possible */
void SCE_Shader_Active (int);
/* versions verbeuses */
void SCE_Shader_Enable (void);
void SCE_Shader_Disable (void);

/* defini le shader comme etant actif (fonction generique) */
void SCE_Shader_Use (SCE_SShader*);

/*
void SCE_Shader_UsePixelShader(SCE_SShader*);

void SCE_Shader_UseVertexShader(SCE_SShader*);
*/

#endif /* guard */
