/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 19/05/2007
   derniere modification : 25/10/2007 */

#include <SCE/SCEMinimal.h>

#include <SCE/utils/SCEResources.h>

#include <SCE/interface/SCEMaterial.h>


/* ajoute le 13/01/2008 */
static void SCE_Material_InitMatTex (SCE_SMatTex *t)
{
    t->tex = NULL;
    t->actived = SCE_TRUE;
    t->canfree = SCE_FALSE;
    t->unit = 0;
}
/* ajoute le 13/01/2008 */
static void SCE_Material_InitMatShd (SCE_SMatShd *s)
{
    s->shd = NULL;
    s->actived = SCE_TRUE;
    s->canfree = SCE_FALSE;
}

/* ajoute le 13/01/2008 */
static void SCE_Material_Init (SCE_SMaterial *m)
{
    m->mat = NULL;
    m->maps = NULL;
    m->shaders = NULL;
}

/* ajoute le 13/01/2008 */
static SCE_SMatTex* SCE_Material_CreateMatTex (void)
{
    SCE_SMatTex *t = SCE_malloc (sizeof *t);
    if (t)
        SCE_Material_InitMatTex (t);
    else
        Logger_LogSrc ();
    return t;
}
/* ajoute le 13/01/2008 */
static SCE_SMatShd* SCE_Material_CreateMatShd (void)
{
    SCE_SMatShd *s = SCE_malloc (sizeof *s);
    if (s)
        SCE_Material_InitMatShd (s);
    else
        Logger_LogSrc ();
    return s;
}

/* ajoute le 13/01/2008 */
static void SCE_Material_DeleteMatTex (void *tex)
{
    if (tex)
    {
        SCE_SMatTex *t = tex;
        if (t->canfree)
            SCE_Texture_Delete (t->tex);
        SCE_free (t);
    }
}
/* ajoute le 13/01/2008 */
static void SCE_Material_DeleteMatShd (void *shd)
{
    if (shd)
    {
        SCE_SMatShd *s = shd;
        if (s->canfree)
            SCE_Shader_Delete (s->shd);
        SCE_free (s);
    }
}

/* revise le 13/01/2008 */
SCE_SMaterial* SCE_Material_Create (void)
{
    SCE_SMaterial *mat = NULL;

    SCE_btstart ();
    mat = SCE_malloc (sizeof *mat);
    if (!mat)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return NULL;
    }

    SCE_Material_Init (mat);

    mat->mat = SCE_CCreateMaterial ();
    if (!mat->mat)
    {
        SCE_free (mat);
        Logger_LogSrc ();
        SCE_btend ();
        return NULL;
    }

    mat->maps = SCE_List_Create (SCE_Material_DeleteMatTex);
    if (!mat->maps)
    {
        SCE_Material_Delete (mat);
        Logger_LogSrc ();
        SCE_btend ();
        return NULL;
    }
    mat->shaders = SCE_List_Create (SCE_Material_DeleteMatShd);
    if (!mat->shaders)
    {
        SCE_Material_Delete (mat);
        Logger_LogSrc ();
        SCE_btend ();
        return NULL;
    }

    SCE_btend ();
    return mat;
}

/* revise le 13/01/2008 */
void SCE_Material_Delete (SCE_SMaterial *mat)
{
    if (mat)
    {
        SCE_List_Delete (mat->shaders);
        SCE_List_Delete (mat->maps);
        SCE_CDeleteMaterial (mat->mat);
    }
}


/* ajoute le 13/01/2008 */
SCE_CMaterial* SCE_Material_GetCMaterial (SCE_SMaterial *mat)
{
    return mat->mat;
}


/* ajoute le 13/01/2008 */
void SCE_Material_SetColor (SCE_SMaterial *mat, SCEenum type,
                            float r, float g, float b, float a)
{
    SCE_CSetMaterialColor (mat->mat, type, r, g, b, a);
}
void SCE_Material_SetColorv (SCE_SMaterial *mat, SCEenum type, float *color)
{
    SCE_CSetMaterialColorv (mat->mat, type, color);
}

float* SCE_Material_GetColor (SCE_SMaterial *mat, SCEenum type)
{
    return SCE_CGetMaterialColor (mat->mat, type);
}
void SCE_Material_GetColorv (SCE_SMaterial *mat, SCEenum type, float *color)
{
    SCE_CGetMaterialColorv (mat->mat, type, color);
}


/* revise le 14/01/2008 */
int SCE_Material_AddTexture (SCE_SMaterial *mat, SCEuint unit,
                             SCE_STexture *tex, int canfree)
{
    SCE_SMatTex *t = NULL;

    SCE_btstart ();
    if (unit >= SCE_CGetMaxTextureUnits ())
    {
        Logger_Log (SCE_INVALID_ARG);
        Logger_LogMsg ("the %dth texture unit is not"
                       "available on your hardware", unit);
        return SCE_ERROR;
    }

#define SCE_matassert(c)\
    if (c)\
    {\
        SCE_Material_DeleteMatTex (t);\
        Logger_LogSrc ();\
        SCE_btend ();\
        return SCE_ERROR;\
    }

    t = SCE_Material_CreateMatTex ();
    SCE_matassert (!t)
    SCE_matassert (SCE_List_AddBack (mat->maps, t) < 0)

    t->tex = tex;
    t->canfree = canfree;
    t->unit = unit;

#undef SCE_matassert

    SCE_btend ();
    return SCE_OK;
}

/* revise le 14/01/2008 */
int SCE_Material_AddShader (SCE_SMaterial *mat, SCE_SShader *shader,
                            int canfree)
{
    SCE_SMatShd *s = NULL;

    SCE_btstart ();
#define SCE_matassert(c)\
    if (c)\
    {\
        SCE_Material_DeleteMatShd (s);\
        Logger_LogSrc ();\
        SCE_btend ();\
        return SCE_ERROR;\
    }

    s = SCE_Material_CreateMatShd ();
    SCE_matassert (!s)
    SCE_matassert (SCE_List_AddBack (mat->shaders, s) < 0)

    s->shd = shader;
    s->canfree = canfree;

#undef SCE_matassert

    SCE_btend ();
    return SCE_OK;
}



/* ajoute le 13/01/2008 */
/* revise le 14/01/2008 */
int SCE_Material_LoadTextureArg (SCE_SMaterial *mat, SCEenum type,
                                 SCEuint unit, int use_mipmap, va_list args)
{
    SCE_STexture *tex = NULL;

    SCE_btstart ();
#define SCE_matassert(c)\
    if (c)\
    {\
        SCE_Texture_Delete (tex);\
        Logger_LogSrc ();\
        SCE_btend ();\
        return SCE_ERROR;\
    }

    tex = SCE_Texture_LoadArg (type, 0, 0, 0, args);
    SCE_matassert (!tex)
    SCE_matassert (SCE_Texture_Build (tex, use_mipmap) < 0)
    SCE_matassert (SCE_Material_AddTexture (mat, unit, tex, SCE_TRUE) < 0)

#undef SCE_matassert

    SCE_btend ();
    return SCE_OK;
}
/* revise le 13/01/2008 */
int SCE_Material_LoadTexture (SCE_SMaterial *mat, SCEenum type, SCEuint unit,
                              int use_mipmap, ...)
{
    va_list args;
    int r;
    va_start (args, use_mipmap);
    r = SCE_Material_LoadTextureArg (mat, type, unit, use_mipmap, args);
    va_end (args);
    return r;
}


int SCE_Material_LoadShader (SCE_SMaterial *mat, const char *vs, const char *ps)
{
    SCE_SShader *shd = NULL;

    SCE_btstart ();
    shd = SCE_Shader_CreateFromFile (vs, ps);
    if (!shd)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }

    if (SCE_Shader_Build (shd) < 0)
    {
        SCE_Shader_Delete (shd);
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }

    if (SCE_Material_AddShader (mat, shd, SCE_TRUE) < 0)
    {
        SCE_Shader_Delete (shd);
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }

    SCE_btend ();
    return SCE_OK;
}

#if 0
/* ajoute le 14/06/2007 */
/* revise le 19/06/2007 */
void SCE_Material_RemoveTexture (SCE_SMaterial *mat, int type, int unit)
{
    
}

/* ajoute le 14/06/2007 */
/* revise le 18/06/2007 */
void SCE_Material_RemoveShader (SCE_SMaterial *mat)
{
    
}


/* ajoute le 14/06/2007 */
/* revise le 19/06/2007 */
/** fonction a tester */
void SCE_Material_ActiveTexture (SCE_SMaterial *mat, int unit, int state)
{
    
}

/* ajoute le 14/06/2007 */
void SCE_Material_ActiveShader (SCE_SMaterial *mat, int state)
{
    
}
#endif


/* revise le 14/01/2008 */
void SCE_Material_Use (SCE_SMaterial *mat)
{
    int i;

    if (mat)
    {
        /**
         * \todo mettre en globale statique ?
         */
        SCE_SMatTex *t = NULL;
        SCE_SMatShd *s = NULL;
        SCE_SListIterator *it = NULL;

        SCE_Shader_Use (NULL);
        for (i=0; i<SCE_CGetMaxTextureUnits (); i++)
            SCE_Texture_Use (NULL, i);

        SCE_List_ForEach (it, mat->maps)
        {
            t = SCE_List_GetData (it);
            if (t->actived)
                SCE_Texture_Use (t->tex, t->unit);
        }

        SCE_List_ForEach (it, mat->shaders)
        {
            s = SCE_List_GetData (it);
            if (s->actived)
                SCE_Shader_Use (s->shd);
        }

        SCE_CUseMaterial (mat->mat);
    }
    else
    {
        SCE_CUseMaterial (NULL);
        SCE_Shader_Use (NULL);

        for (i=0; i<SCE_CGetMaxTextureUnits (); i++)
            SCE_Texture_Use (NULL, i);
    }
}
